Package: rlang
Version: 1.1.6
Title: Functions for Base Types and Core R and 'Tidyverse' Features
Description: A toolbox for working with base types, core R features
  like the condition system, and core 'Tidyverse' features like tidy
  evaluation.
Authors@R: c(
    person("Lionel", "Henry", ,"lionel@posit.co", c("aut", "cre")),
    person("Hadley", "Wickham", ,"hadley@posit.co", "aut"),
    person(given = "mikefc",
           email = "mikefc@coolbutuseless.com",
           role = "cph",
           comment = "Hash implementation based on Mike's xxhashlite"),
    person(given = "Yann",
           family = "Collet",
           role = "cph",
           comment = "Author of the embedded xxHash library"),
    person(given = "Posit, PBC", role = c("cph", "fnd"))
    )
License: MIT + file LICENSE
ByteCompile: true
Biarch: true
Depends: R (>= 3.5.0)
Imports: utils
Suggests: cli (>= 3.1.0), covr, crayon, desc, fs, glue, knitr,
        magrittr, methods, pillar, pkgload, rmarkdown, stats, testthat
        (>= 3.2.0), tibble, usethis, vctrs (>= 0.2.3), withr
Enhances: winch
Encoding: UTF-8
RoxygenNote: 7.3.2
URL: https://rlang.r-lib.org, https://github.com/r-lib/rlang
BugReports: https://github.com/r-lib/rlang/issues
Config/build/compilation-database: true
Config/testthat/edition: 3
Config/Needs/website: dplyr, tidyverse/tidytemplate
NeedsCompilation: yes
Packaged: 2025-04-10 09:25:27 UTC; lionel
Author: Lionel Henry [aut, cre],
  Hadley Wickham [aut],
  mikefc [cph] (Hash implementation based on Mike's xxhashlite),
  Yann Collet [cph] (Author of the embedded xxHash library),
  Posit, PBC [cph, fnd]
Maintainer: Lionel Henry <lionel@posit.co>
Repository: CRAN
Date/Publication: 2025-04-11 08:40:10 UTC
Built: R 4.4.0; amd64-portbld-midnightbsd4; 2026-07-17 17:25:16 UTC; unix
