Package: rlanglibtest
Title: What the Package Does (one line, title case)
Version: 0.0.0.9000
Authors@R: 'Lionel Henry <lionel.hry@gmail.com> [aut, cre]'
Description: What the package does (one paragraph).
Depends:
    R (>= 3.1.0)
Imports:
    rlang
LinkingTo:
    rlang
Encoding: UTF-8
LazyData: true
RoxygenNote: 6.0.1
