# Overviews ---------------------------------------------------------------

#' `r title("topic_data_mask")`
#'
#' ```{r, child = "man/rmd/topic-data-mask.Rmd"}
#' ```
#'
#' @keywords internal
#' @name topic-data-mask
NULL

#' `r title("topic_data_mask_programming")`
#'
#' ```{r, child = "man/rmd/topic-data-mask-programming.Rmd"}
#' ```
#'
#' @keywords internal
#' @name topic-data-mask-programming
NULL

#' `r title("topic_metaprogramming")`
#'
#' ```{r, child = "man/rmd/topic-metaprogramming.Rmd"}
#' ```
#'
#' @keywords internal
#' @name topic-metaprogramming
NULL

#' `r title("topic_defuse")`
#'
#' ```{r, child = "man/rmd/topic-defuse.Rmd"}
#' ```
#'
#' @keywords internal
#' @aliases quotation nse-defuse
#' @name topic-defuse
NULL

#' `r title("topic_inject")`
#'
#' ```{r, child = "man/rmd/topic-inject.Rmd"}
#' ```
#'
#' @keywords internal
#' @aliases quasiquotation nse-force nse-inject
#' @name topic-inject
NULL

#' `r title("topic_quosure")`
#'
#' ```{r, child = "man/rmd/topic-quosure.Rmd"}
#' ```
#'
#' @keywords internal
#' @name topic-quosure
NULL


# Guides ------------------------------------------------------------------

#' `r title("topic_data_mask_ambiguity")`
#'
#' ```{r, child = "man/rmd/topic-data-mask-ambiguity.Rmd"}
#' ```
#'
#' @keywords internal
#' @name topic-data-mask-ambiguity
NULL

#' `r title("topic_double_evaluation")`
#'
#' ```{r, child = "man/rmd/topic-double-evaluation.Rmd"}
#' ```
#'
#' @keywords internal
#' @name topic-double-evaluation
NULL

#' `r title("topic_multiple_columns")`
#'
#' ```{r, child = "man/rmd/topic-multiple-columns.Rmd"}
#' ```
#'
#' @keywords internal
#' @name topic-multiple-columns
NULL


# Notes -------------------------------------------------------------------

#' `r title("topic_embrace_non_args")`
#'
#' ```{r, child = "man/rmd/topic-embrace-non-args.Rmd"}
#' ```
#'
#' @keywords internal
#' @name topic-embrace-non-args
NULL

#' `r title("topic_embrace_constants")`
#'
#' ```{r, child = "man/rmd/topic-embrace-constants.Rmd"}
#' ```
#'
#' @keywords internal
#' @name topic-embrace-constants
NULL

#' `r title("topic_inject_out_of_context")`
#'
#' ```{r, child = "man/rmd/topic-inject-out-of-context.Rmd"}
#' ```
#'
#' @keywords internal
#' @name topic-inject-out-of-context
NULL
