# as_box_if() ensures boxed value if predicate returns TRUE

    Code
      as_box_if(NULL, ~10)
    Condition <rlang_error>
      Error in `.p()`:
      ! Predicate functions must return a single `TRUE` or `FALSE`, not the number 10
    Code
      as_box_if(NULL, ~ c(TRUE, FALSE))
    Condition <rlang_error>
      Error in `.p()`:
      ! Predicate functions must return a single `TRUE` or `FALSE`, not a logical vector

