% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weakref.R
\name{is_weakref}
\alias{is_weakref}
\title{Is object a weak reference?}
\usage{
is_weakref(x)
}
\arguments{
\item{x}{An object to test.}
}
\description{
Is object a weak reference?
}
