% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/obj.R
\name{obj_address}
\alias{obj_address}
\title{Address of an R object}
\usage{
obj_address(x)
}
\arguments{
\item{x}{Any R object.}
}
\value{
Its address in memory in a string.
}
\description{
Address of an R object
}
\keyword{internal}
