% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/node.R
\name{new_call}
\alias{new_call}
\title{Create a new call from components}
\usage{
new_call(car, cdr = NULL)
}
\arguments{
\item{car}{The head of the call. It should be a
\link[=is_callable]{callable} object: a symbol, call, or literal
function.}

\item{cdr}{The tail of the call, i.e. a \link[=new_node]{pairlist} of
arguments.}
}
\description{
Create a new call from components
}
\keyword{internal}
